@@ -2,8 +2,10 @@ from random import Random
 
 from sampo.generator import SyntheticGraphType
 from sampo.generator.environment import get_contractor
+from sampo.generator.environment.landscape import get_landscape_by_wg
 from sampo.generator.pipeline.extension import extend_names, extend_resources
 from sampo.generator.pipeline.project import get_small_graph, get_graph
+from sampo.schemas import LandscapeConfiguration
 from sampo.schemas.graph import WorkGraph
 
 
@@ -71,3 +73,6 @@ class SimpleSynthetic:
         wg = extend_names(uniq_works, wg, self._rand)
         wg = extend_resources(uniq_resources, wg, self._rand)
         return wg
+
+    def simple_synthetic_landscape(self, wg: WorkGraph) -> LandscapeConfiguration:
+        return get_landscape_by_wg(wg, self._rand)
