@@ -2,7 +2,7 @@ from enum import Enum
 
 
 class LagOptimizationStrategy(Enum):
-    TRUE = True,
-    FALSE = False,
-    AUTO = None,
+    TRUE = True
+    FALSE = False
+    AUTO = None
     NONE = None
