@@ -2,5 +2,6 @@ from sampo.scheduler.base import Scheduler, SchedulerType
 from sampo.scheduler.genetic import GeneticScheduler
 from sampo.scheduler.generic import GenericScheduler
 from sampo.scheduler.heft import HEFTScheduler, HEFTBetweenScheduler
-from sampo.scheduler.topological import TopologicalScheduler
+from sampo.scheduler.topological import TopologicalScheduler, RandomizedTopologicalScheduler
+from sampo.scheduler.lft import LFTScheduler, RandomizedLFTScheduler
 from sampo.scheduler.native_wrapper import NativeWrapper
