@@ -1,11 +1,12 @@
 from typing import Type, Callable, Iterable
 
 from sampo.scheduler.base import Scheduler, SchedulerType
+from sampo.scheduler.utils import WorkerContractorPool, get_worker_contractor_pool
 from sampo.scheduler.utils.time_computaion import calculate_working_time_cascade
 from sampo.scheduler.resource.base import ResourceOptimizer
 from sampo.scheduler.timeline.base import Timeline
 from sampo.scheduler.utils.multi_contractor import run_contractor_search, get_worker_borders
-from sampo.schemas.contractor import Contractor, get_worker_contractor_pool, WorkerContractorPool
+from sampo.schemas.contractor import Contractor
 from sampo.schemas.graph import WorkGraph, GraphNode
 from sampo.schemas.landscape import LandscapeConfiguration
 from sampo.schemas.resources import Worker
@@ -18,13 +19,20 @@ from sampo.utilities.validation import validate_schedule
 
 
 # TODO Кажется, это не работает - лаги не учитываются
-def get_finish_time_default(node, worker_team, node2swork, spec, assigned_parent_time, timeline, work_estimator):
+def get_finish_time_default(node, worker_team, node2swork, spec, assigned_parent_time, timeline,
+                            work_estimator) -> Time:
     return timeline.find_min_start_time(node, worker_team, node2swork, spec,
                                         assigned_parent_time, work_estimator) \
         + calculate_working_time_cascade(node, worker_team,
                                          work_estimator)  # TODO Кажется, это не работает - лаги не учитываются
 
 
+PRIORITIZATION_F = Callable[[WorkGraph, WorkTimeEstimator], list[GraphNode]]
+RESOURCE_OPTIMIZE_F = Callable[[GraphNode, list[Contractor], WorkSpec, WorkerContractorPool,
+                                dict[GraphNode, ScheduledWork], Time, Timeline, WorkTimeEstimator],
+                               tuple[Time, Time, Contractor, list[Worker]]]
+
+
 class GenericScheduler(Scheduler):
     """
     Implementation of a universal scheme of scheduler.
@@ -36,10 +44,8 @@ class GenericScheduler(Scheduler):
                  scheduler_type: SchedulerType,
                  resource_optimizer: ResourceOptimizer,
                  timeline_type: Type,
-                 prioritization_f: Callable[[WorkGraph, WorkTimeEstimator], list[GraphNode]],
-                 optimize_resources_f: Callable[[GraphNode, list[Contractor], WorkSpec, WorkerContractorPool,
-                                                 dict[GraphNode, ScheduledWork], Time, Timeline, WorkTimeEstimator],
-                                                 tuple[Time, Time, Contractor, list[Worker]]],
+                 prioritization_f: PRIORITIZATION_F,
+                 optimize_resources_f: RESOURCE_OPTIMIZE_F,
                  work_estimator: WorkTimeEstimator = DefaultWorkEstimator()):
         super().__init__(scheduler_type, resource_optimizer, work_estimator)
         self._timeline_type = timeline_type
@@ -49,24 +55,33 @@ class GenericScheduler(Scheduler):
     def get_default_res_opt_function(self, get_finish_time=get_finish_time_default) \
             -> Callable[[GraphNode, list[Contractor], WorkSpec, WorkerContractorPool,
                          dict[GraphNode, ScheduledWork], Time, Timeline, WorkTimeEstimator],
-                         tuple[Time, Time, Contractor, list[Worker]]]:
+            tuple[Time, Time, Contractor, list[Worker]]]:
+        """
+        Here is default resource optimization getter function.
+
+        Constructs function that receives node with necessary scheduling inner info and
+        returns start time, finish time and assigned workers for that node.
+
+        :param get_finish_time: function that
+        :return: resource optimization function that can be passed as argument when constructing `GenericScheduler`.
+        """
 
         def optimize_resources_def(node: GraphNode, contractors: list[Contractor], spec: WorkSpec,
                                    worker_pool: WorkerContractorPool, node2swork: dict[GraphNode, ScheduledWork],
                                    assigned_parent_time: Time, timeline: Timeline, work_estimator: WorkTimeEstimator):
+            def ft_getter(worker_team) -> Time:
+                return get_finish_time(node, worker_team, node2swork, spec,
+                                       assigned_parent_time, timeline, work_estimator)
+
             def run_with_contractor(contractor: Contractor) -> tuple[Time, Time, list[Worker]]:
                 min_count_worker_team, max_count_worker_team, workers \
                     = get_worker_borders(worker_pool, contractor, node.work_unit.worker_reqs)
 
                 if len(workers) != len(node.work_unit.worker_reqs):
-                    return Time(0), Time.inf(), []
+                    return assigned_parent_time, Time.inf(), []
 
                 workers = [worker.copy() for worker in workers]
 
-                def ft_getter(worker_team):
-                    return get_finish_time(node, worker_team, node2swork, spec,
-                                           assigned_parent_time, timeline, work_estimator)
-
                 # apply worker team spec
                 self.optimize_resources_using_spec(node.work_unit, workers, spec,
                                                    lambda optimize_array: self.resource_optimizer.optimize_resources(
@@ -86,16 +101,16 @@ class GenericScheduler(Scheduler):
     def schedule_with_cache(self,
                             wg: WorkGraph,
                             contractors: list[Contractor],
-                            landscape: LandscapeConfiguration() = LandscapeConfiguration(),
                             spec: ScheduleSpec = ScheduleSpec(),
                             validate: bool = False,
                             assigned_parent_time: Time = Time(0),
-                            timeline: Timeline | None = None) \
-            -> tuple[Schedule, Time, Timeline, list[GraphNode]]:
+                            timeline: Timeline | None = None,
+                            landscape: LandscapeConfiguration() = LandscapeConfiguration()) \
+            -> list[tuple[Schedule, Time, Timeline, list[GraphNode]]]:
         ordered_nodes = self.prioritization(wg, self.work_estimator)
 
         schedule, schedule_start_time, timeline = \
-            self.build_scheduler(wg, ordered_nodes, contractors, landscape, spec, self.work_estimator,
+            self.build_scheduler(ordered_nodes, contractors, landscape, spec, self.work_estimator,
                                  assigned_parent_time, timeline)
         schedule = Schedule.from_scheduled_works(
             schedule,
@@ -105,10 +120,9 @@ class GenericScheduler(Scheduler):
         if validate:
             validate_schedule(schedule, wg, contractors)
 
-        return schedule, schedule_start_time, timeline, ordered_nodes
+        return [(schedule, schedule_start_time, timeline, ordered_nodes)]
 
     def build_scheduler(self,
-                        wg: WorkGraph,
                         ordered_nodes: list[GraphNode],
                         contractors: list[Contractor],
                         landscape: LandscapeConfiguration = LandscapeConfiguration(),
