@@ -1,9 +1,10 @@
 from abc import ABC, abstractmethod
 from typing import Optional
 
+from sampo.schemas import MaterialDelivery
 from sampo.schemas.contractor import Contractor
 from sampo.schemas.graph import GraphNode
-from sampo.schemas.resources import Worker
+from sampo.schemas.resources import Worker, Material
 from sampo.schemas.schedule_spec import WorkSpec
 from sampo.schemas.scheduled_work import ScheduledWork
 from sampo.schemas.time import Time
@@ -88,3 +89,23 @@ class Timeline(ABC):
                         worker_team: list[Worker],
                         spec: WorkSpec):
         ...
+
+
+class BaseSupplyTimeline(ABC):
+    @abstractmethod
+    def can_schedule_at_the_moment(self, node: GraphNode, start_time: Time,
+                                   materials: list[Material]) -> bool:
+        ...
+
+    @abstractmethod
+    def find_min_material_time(self, node: GraphNode, start_time: Time,
+                               materials: list[Material]) -> Time:
+        ...
+
+    @abstractmethod
+    def deliver_resources(self,
+                          node: GraphNode,
+                          deadline: Time,
+                          materials: list[Material],
+                          update: bool = False) -> tuple[MaterialDelivery, Time]:
+        ...
\ No newline at end of file
