@@ -1,9 +1,10 @@
 from typing import Optional
 
 from sampo.scheduler.timeline.base import Timeline
-from sampo.scheduler.timeline.material_timeline import SupplyTimeline
+from sampo.scheduler.timeline.from_start_material_timeline import FromStartSupplyTimeline
 from sampo.scheduler.timeline.zone_timeline import ZoneTimeline
-from sampo.schemas.contractor import Contractor, WorkerContractorPool
+from sampo.scheduler.utils import WorkerContractorPool
+from sampo.schemas import Contractor
 from sampo.schemas.graph import GraphNode
 from sampo.schemas.landscape import LandscapeConfiguration
 from sampo.schemas.resources import Worker
@@ -27,7 +28,7 @@ class JustInTimeTimeline(Timeline):
             for worker_offer in worker_offers.values():
                 self._timeline[worker_offer.get_agent_id()] = [(Time(0), worker_offer.count)]
 
-        self._material_timeline = SupplyTimeline(landscape)
+        self._material_timeline = FromStartSupplyTimeline(landscape)
         self.zone_timeline = ZoneTimeline(landscape.zone_config)
 
     def find_min_start_time_with_additional(self, node: GraphNode,
@@ -54,9 +55,9 @@ class JustInTimeTimeline(Timeline):
         """
         # if current job is the first
         if not node2swork:
-            max_material_time = self._material_timeline.find_min_material_time(node.id, assigned_parent_time,
-                                                                               node.work_unit.need_materials(),
-                                                                               node.work_unit.workground_size)
+            max_material_time = self._material_timeline.find_min_material_time(node,
+                                                                               assigned_parent_time,
+                                                                               node.work_unit.need_materials())
 
             max_zone_time = self.zone_timeline.find_min_start_time(node.work_unit.zone_reqs, max_material_time, Time(0))
 
@@ -65,7 +66,50 @@ class JustInTimeTimeline(Timeline):
         max_parent_time = max(node.min_start_time(node2swork), assigned_parent_time)
         # define the max agents time when all needed workers are off from previous tasks
         max_agent_time = Time(0)
+        cur_start_time = max_agent_time
 
+        inseparable_chain = node.get_inseparable_chain_with_self()
+
+        new_finish_time = cur_start_time
+        for dep_node in inseparable_chain:
+            # set start time as finish time of original work
+            # set finish time as finish time + working time of current node with identical resources
+            # (the same as in original work)
+            # set the same workers on it
+            # TODO Decide where this should be
+            dep_parent_time = dep_node.min_start_time(node2swork)
+
+            dep_st = max(new_finish_time, dep_parent_time)
+            working_time = work_estimator.estimate_time(dep_node.work_unit, worker_team)
+            new_finish_time = dep_st + working_time
+
+        exec_time = new_finish_time - cur_start_time
+
+        found_earliest_time = False
+        while not found_earliest_time:
+            material_time = self._material_timeline.find_min_material_time(node,
+                                                                           cur_start_time,
+                                                                           node.work_unit.need_materials())
+
+            cur_start_time = self._find_min_start_time(worker_team, cur_start_time, spec)
+
+            if cur_start_time > material_time:
+                continue
+
+            zone_time = self.zone_timeline.find_min_start_time(node.work_unit.zone_reqs, cur_start_time,
+                                                               exec_time)
+            if zone_time > cur_start_time:
+                cur_start_time = zone_time
+            else:
+                found_earliest_time = True
+
+        c_st = cur_start_time
+
+        c_ft = c_st + exec_time
+        return c_st, c_ft, None
+
+    def _find_min_start_time(self, worker_team: list[Worker], _max_agent_time: Time, spec: WorkSpec):
+        max_agent_time = _max_agent_time
         if spec.is_independent:
             # grab from the end
             for worker in worker_team:
@@ -88,34 +132,7 @@ class JustInTimeTimeline(Timeline):
                     needed_count -= offer_count
                     ind -= 1
 
-        c_st = max(max_agent_time, max_parent_time)
-
-        new_finish_time = c_st
-        for dep_node in node.get_inseparable_chain_with_self():
-            # set start time as finish time of original work
-            # set finish time as finish time + working time of current node with identical resources
-            # (the same as in original work)
-            # set the same workers on it
-            # TODO Decide where this should be
-            dep_parent_time = dep_node.min_start_time(node2swork)
-
-            dep_st = max(new_finish_time, dep_parent_time)
-            working_time = work_estimator.estimate_time(dep_node.work_unit, worker_team)
-            new_finish_time = dep_st + working_time
-
-        new_finish_time -= 1
-        exec_time = new_finish_time - c_st
-
-        max_material_time = self._material_timeline.find_min_material_time(node.id, c_st,
-                                                                           node.work_unit.need_materials(),
-                                                                           node.work_unit.workground_size)
-
-        max_zone_time = self.zone_timeline.find_min_start_time(node.work_unit.zone_reqs, c_st, exec_time)
-
-        c_st = max(c_st, max_material_time, max_zone_time)
-
-        c_ft = c_st + exec_time
-        return c_st, c_ft, None
+        return max_agent_time
 
     def can_schedule_at_the_moment(self,
                                    node: GraphNode,
@@ -129,7 +146,7 @@ class JustInTimeTimeline(Timeline):
             for worker in worker_team:
                 worker_timeline = self._timeline[(worker.contractor_id, worker.name)]
                 last_cpkt_time, _ = worker_timeline[0]
-                if last_cpkt_time >= start_time:
+                if last_cpkt_time > start_time:
                     return False
             return True
         else:
@@ -138,7 +155,7 @@ class JustInTimeTimeline(Timeline):
                 for p in dep_node.parents:
                     if p != dep_node.inseparable_parent:
                         swork = node2swork.get(p, None)
-                        if swork is None or swork.finish_time >= start_time:
+                        if swork is None or swork.finish_time > start_time:
                             return False
 
             max_agent_time = Time(0)
@@ -159,9 +176,8 @@ class JustInTimeTimeline(Timeline):
             if not max_agent_time <= start_time:
                 return False
 
-            if not self._material_timeline.can_schedule_at_the_moment(node.id, start_time,
-                                                                      node.work_unit.need_materials(),
-                                                                      node.work_unit.workground_size):
+            if not self._material_timeline.can_schedule_at_the_moment(node, start_time,
+                                                                      node.work_unit.need_materials()):
                 return False
             if not self.zone_timeline.can_schedule_at_the_moment(node.work_unit.zone_reqs, start_time, exec_time):
                 return False
@@ -285,12 +301,14 @@ class JustInTimeTimeline(Timeline):
                 lag, working_time = 0, work_estimator.estimate_time(node.work_unit, workers)
             c_st = max(c_ft + lag, max_parent_time)
 
-            new_finish_time = c_st + working_time
+            deliveries, mat_del_time = self._material_timeline.deliver_resources(dep_node,
+                                                                                 c_st,
+                                                                                 dep_node.work_unit.need_materials(),
+                                                                                 True)
 
-            deliveries, _, new_finish_time = self._material_timeline.deliver_materials(dep_node.id, c_st,
-                                                                                       new_finish_time,
-                                                                                       dep_node.work_unit.need_materials(),
-                                                                                       dep_node.work_unit.workground_size)
+            c_st = max(mat_del_time, c_st)
+
+            new_finish_time = c_st + working_time
 
             node2swork[dep_node] = ScheduledWork(work_unit=dep_node.work_unit,
                                                  start_end_time=(c_st, new_finish_time),
