@@ -1,13 +1,11 @@
-from copy import deepcopy
 from dataclasses import dataclass
-from typing import Any
+from typing import Any, Callable
 
 from sampo.schemas.contractor import Contractor
 from sampo.schemas.landscape import MaterialDelivery
 from sampo.schemas.resources import Equipment, ConstructionObject, Worker
 from sampo.schemas.serializable import AutoJSONSerializable
 from sampo.schemas.time import Time
-from sampo.schemas.time_estimator import WorkTimeEstimator
 from sampo.schemas.works import WorkUnit
 from sampo.schemas.zones import ZoneTransition
 from sampo.utilities.serializers import custom_serializer
@@ -37,7 +35,7 @@ class ScheduledWork(AutoJSONSerializable['ScheduledWork']):
                  equipments: list[Equipment] | None = None,
                  zones_pre: list[ZoneTransition] | None = None,
                  zones_post: list[ZoneTransition] | None = None,
-                 materials: list[MaterialDelivery] | None = None,
+                 materials: MaterialDelivery | None = None,
                  c_object: ConstructionObject | None = None):
         self.id = work_unit.id
         self.name = work_unit.name
@@ -50,7 +48,7 @@ class ScheduledWork(AutoJSONSerializable['ScheduledWork']):
         self.equipments = equipments if equipments is not None else []
         self.zones_pre = zones_pre if zones_pre is not None else []
         self.zones_post = zones_post if zones_post is not None else []
-        self.materials = materials if materials is not None else []
+        self.materials = materials if materials is not None else MaterialDelivery('')
         self.object = c_object if c_object is not None else []
 
         if contractor is not None:
@@ -63,31 +61,35 @@ class ScheduledWork(AutoJSONSerializable['ScheduledWork']):
 
         self.cost = sum([worker.get_cost() * self.duration.value for worker in self.workers])
 
-    def __str__(self):
+    def __str__(self) -> str:
         return f'ScheduledWork[work_unit={self.id}, start_end_time={self.start_end_time}, ' \
                f'workers={self.workers}, contractor={self.contractor}]'
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return self.__str__()
 
     @custom_serializer('workers')
     @custom_serializer('zones_pre')
     @custom_serializer('zones_post')
     @custom_serializer('start_end_time')
-    def serialize_serializable_list(self, value):
+    def serialize_serializable_list(self, value) -> list:
         return [t._serialize() for t in value]
 
     @classmethod
     @custom_serializer('start_end_time', deserializer=True)
-    def deserialize_time(cls, value):
-        return [Time._deserialize(t) for t in value]
+    def deserialize_time(cls, value) -> tuple[Time, Time]:
+        return Time._deserialize(value[0]), Time._deserialize(value[1])
 
     @classmethod
     @custom_serializer('workers', deserializer=True)
+    def deserialize_workers(cls, value) -> list[Worker]:
+        return [Worker._deserialize(t) for t in value]
+
+    @classmethod
     @custom_serializer('zones_pre', deserializer=True)
     @custom_serializer('zones_post', deserializer=True)
-    def deserialize_workers(cls, value):
-        return [Worker._deserialize(t) for t in value]
+    def deserialize_zone_transitions(cls, value) -> list[ZoneTransition]:
+        return [ZoneTransition._deserialize(t) for t in value]
 
     @property
     def start_time(self) -> Time:
@@ -106,11 +108,11 @@ class ScheduledWork(AutoJSONSerializable['ScheduledWork']):
         self.start_end_time = (self.start_end_time[0], val)
 
     @staticmethod
-    def start_time_getter():
+    def start_time_getter() -> Callable[[], Time]:
         return lambda x: x.start_end_time[0]
 
     @staticmethod
-    def finish_time_getter():
+    def finish_time_getter() -> Callable[[], Time]:
         return lambda x: x.start_end_time[1]
 
     @property
