@@ -2,11 +2,12 @@ from abc import ABC, abstractmethod
 from enum import Enum
 from operator import attrgetter
 from random import Random
-from typing import Optional
+from typing import Optional, Type
 
 import numpy.random
 import math
 
+from sampo.schemas import Interval, IntervalGaussian
 from sampo.schemas.requirements import WorkerReq
 from sampo.schemas.resources import Worker
 from sampo.schemas.resources import WorkerProductivityMode
@@ -43,6 +44,10 @@ class WorkTimeEstimator(ABC):
     def estimate_time(self, work_unit: WorkUnit, worker_list: list[Worker]):
         ...
 
+    @abstractmethod
+    def get_recreate_info(self) -> tuple[Type, tuple]:
+        ...
+
 
 class DefaultWorkEstimator(WorkTimeEstimator):
 
@@ -52,6 +57,8 @@ class DefaultWorkEstimator(WorkTimeEstimator):
         self._estimation_mode = WorkEstimationMode.Realistic
         self.rand = rand
         self._productivity_mode = WorkerProductivityMode.Static
+        self._productivity = {worker: {'__ALL__': IntervalGaussian(1, 0.2, 1, 0)}
+                              for worker in ['driver', 'fitter', 'manager', 'handyman', 'electrician', 'engineer']}
 
     def find_work_resources(self, work_name: str, work_volume: float, resource_name: list[str] | None = None) \
             -> list[WorkerReq]:
@@ -59,7 +66,7 @@ class DefaultWorkEstimator(WorkTimeEstimator):
             resource_name = ['driver', 'fitter', 'manager', 'handyman', 'electrician', 'engineer']
         dist = numpy.random.poisson(work_volume * 3, len(resource_name))
         return [WorkerReq(kind=name,
-                          volume=Time(int(work_volume * numpy.random.poisson(work_volume ** 0.5, 1)[0])),
+                          volume=Time(math.ceil(work_volume * numpy.random.poisson(work_volume ** 0.5, 1)[0])),
                           min_count=int(dist[i]),
                           max_count=int(dist[i] * 2))
                 for i, name in enumerate(resource_name)]
@@ -86,27 +93,35 @@ class DefaultWorkEstimator(WorkTimeEstimator):
             worker_count = 0 if worker is None else worker.count
             if worker_count < req.min_count:
                 return Time.inf()
-            productivity = DefaultWorkEstimator.get_productivity_of_worker(worker, self.rand, req.max_count,
-                                                                           self._productivity_mode) / worker_count
+            productivity = self._get_productivity(worker, req.max_count)
             if productivity == 0:
                 return Time.inf()
             times.append(Time(math.ceil(req.volume / productivity)))
         return max(max(times), Time(0))
 
-    @staticmethod
-    def get_productivity_of_worker(worker: Worker, rand: Optional[Random] = None, max_groups: int = 0,
-                                   productivity_mode: WorkerProductivityMode = WorkerProductivityMode.Static):
+    def _get_productivity(self, worker: Worker, max_count_workers: int) -> float:
         """
         Calculate the productivity of the Worker
-        It has 2 mods: stochastic and non-stochastic, depending on the value of rand
-
-        :param productivity_mode:
-        :param max_groups:
+        It has 2 modes: stochastic and non-stochastic, depending on the value of rand
         :param worker: the worker
-        :param rand: parameter for stochastic part
-        :return: productivity of received worker
+        :param max_count_workers: maximum workers count according to worker reqs
+        :return:
         """
-        return worker.get_productivity(rand, productivity_mode) * communication_coefficient(worker.count, max_groups)
+        worker_productivities = self._productivity[worker.name]
+        productivity_interval = worker_productivities.get(worker.contractor_id, worker_productivities['__ALL__'])
+        productivity = productivity_interval.mean \
+            if self._productivity_mode is WorkerProductivityMode.Static \
+            else productivity_interval.rand_float(self.rand)
+        return productivity * worker.count * communication_coefficient(worker.count, max_count_workers)
+
+    def set_worker_productivity(self, productivity: Interval, name: str, contractor: str | None = None):
+        if contractor is None:
+            self._productivity[name]['__ALL__'] = productivity
+            return
+        self._productivity[name][contractor] = productivity
+
+    def get_recreate_info(self) -> tuple[Type, tuple]:
+        return DefaultWorkEstimator, ()
 
 
 def communication_coefficient(groups_count: int, max_groups: int) -> float:
