@@ -21,7 +21,7 @@ class WorkUnit(AutoJSONSerializable['WorkUnit'], Identifiable):
                  object_reqs: list[ConstructionObjectReq] = None,
                  zone_reqs: list[ZoneReq] = None,
                  description: str = '',
-                 group: str = 'default',
+                 group: str = 'main project',
                  is_service_unit: bool = False,
                  volume: float = 0,
                  volume_type: str = 'unit',
@@ -59,10 +59,9 @@ class WorkUnit(AutoJSONSerializable['WorkUnit'], Identifiable):
         self.description = description
         self.group = group
         self.is_service_unit = is_service_unit
-        self.volume = volume
+        self.volume = float(volume)
         self.volume_type = volume_type
         self.display_name = display_name if display_name else name
-        self.workground_size = workground_size
 
     def __del__(self):
         for attr in self.__dict__.values():
@@ -72,18 +71,32 @@ class WorkUnit(AutoJSONSerializable['WorkUnit'], Identifiable):
         return [req.material() for req in self.material_reqs]
 
     @custom_serializer('worker_reqs')
-    def worker_reqs_serializer(self, value: list[WorkerReq]):
+    @custom_serializer('zone_reqs')
+    @custom_serializer('material_reqs')
+    def serialize_serializable_list(self, value) -> list:
         """
-        Return serialized list of worker requirements
+        Return serialized list of values.
+        Values should be serializable.
 
-        :param value: list of worker requirements
-        :return: list of worker requirements
+        :param value: list of values
+        :return: list of serialized values
         """
-        return [wr._serialize() for wr in value]
+        return [t._serialize() for t in value]
+
+    @classmethod
+    @custom_serializer('material_reqs', deserializer=True)
+    def material_reqs_deserializer(cls, value) -> list[MaterialReq]:
+        """
+        Get list of material requirements
+
+        :param value: serialized list of material requirements
+        :return: list of material requirements
+        """
+        return [MaterialReq._deserialize(wr) for wr in value]
 
     @classmethod
     @custom_serializer('worker_reqs', deserializer=True)
-    def worker_reqs_deserializer(cls, value):
+    def worker_reqs_deserializer(cls, value) -> list[WorkerReq]:
         """
         Get list of worker requirements
 
@@ -92,26 +105,27 @@ class WorkUnit(AutoJSONSerializable['WorkUnit'], Identifiable):
         """
         return [WorkerReq._deserialize(wr) for wr in value]
 
-    @custom_serializer('zone_reqs')
-    def zone_reqs_serializer(self, value: list[WorkerReq]):
-        """
-        Return serialized list of worker requirements
-
-        :param value: list of worker requirements
-        :return: list of worker requirements
-        """
-        return [wr._serialize() for wr in value]
-
     @classmethod
     @custom_serializer('zone_reqs', deserializer=True)
-    def zone_reqs_deserializer(cls, value):
+    def zone_reqs_deserializer(cls, value) -> list[ZoneReq]:
         """
         Get list of worker requirements
 
         :param value: serialized list of work requirements
         :return: list of worker requirements
         """
-        return [WorkerReq._deserialize(wr) for wr in value]
+        return [ZoneReq._deserialize(wr) for wr in value]
+
+    @classmethod
+    @custom_serializer('material_reqs', deserializer=True)
+    def material_reqs_deserializer(cls, value) -> list[MaterialReq]:
+        """
+        Get list of material requirements
+
+        :param value: serialized list of material requirements
+        :return: list of material requirements
+        """
+        return [MaterialReq._deserialize(wr) for wr in value]
 
     def __getstate__(self):
         # custom method to avoid calling __hash__() on GraphNode objects
@@ -131,4 +145,3 @@ class WorkUnit(AutoJSONSerializable['WorkUnit'], Identifiable):
         self.volume_type = new_work_unit.volume_type
         self.group = new_work_unit.group
         self.display_name = new_work_unit.display_name
-        self.workground_size = new_work_unit.workground_size
