@@ -24,9 +24,8 @@ class CSVParser:
                         sep_wg: str = ';',
                         sep_history: str = ';',
                         name_mapper: NameMapper | None = None,
-                        change_all_info: bool = False,
-                        change_only_omitted_info_and_connections: bool = False,
-                        change_all_info_and_connections: bool = False) -> pd.DataFrame:
+                        all_connections: bool = False,
+                        change_connections_info: bool = False) -> pd.DataFrame:
         """
         Read the input data about work graph and preprocess it.
 
@@ -70,7 +69,7 @@ class CSVParser:
         :param name_mapper: name mapper that translates 'activity_name' to the name, as from a document
         :param project_info: path to the works' info file
         :param history_data: path to the history data of connection file
-        :param change_connections: whether it is necessary to change connections
+        :param all_connections: whether it is necessary to change connections
         :param change_connections_info: whether it is necessary to change connections' information based on history data?
         :param sep_wg: separating character. It's mandatory if you send the WorkGraph .csv
         :param sep_history: separating character. It's mandatory if you send the HistoryData .csv file path
@@ -85,33 +84,58 @@ class CSVParser:
             raise InputDataException(
                 'you have neither history data about tasks nor tasks\' connection info in received .csv file.')
 
-        temp_lst = [math.nan] * graph_df.shape[0]
-        if 'predecessor_ids' not in graph_df.columns:
-            for col in ['predecessor_ids', 'connection_types', 'lags']:
-                graph_df[col] = temp_lst
-            # if we ought to restore predecessor info from history data
-            graph_df = preprocess_graph_df(graph_df, name_mapper)
-            works_info = set_connections_info(graph_df, history_df, mapper=name_mapper,
-                                              change_connections_info=True)
-        else:
-            for col in ['connection_types', 'lags']:
-                if col not in graph_df.columns:
-                    graph_df[col] = temp_lst
-            graph_df = preprocess_graph_df(graph_df, name_mapper)
-            if change_all_info:
-                works_info = set_connections_info(graph_df, history_df, mapper=name_mapper)
-            elif change_only_omitted_info_and_connections:
-                works_info = set_connections_info(graph_df, history_df, mapper=name_mapper,
-                                                  change_connections_info=True,
-                                                  expert_connections_info=True)
-            elif change_all_info_and_connections:
-                works_info = set_connections_info(graph_df, history_df, change_connections_info=True)
-            else:
-                works_info = set_connections_info(graph_df, history_df, mapper=name_mapper,
-                                                  expert_connections_info=True)
+        graph_df = preprocess_graph_df(graph_df, name_mapper)
+        id2ind = {graph_df.loc[i, 'activity_id']: i for i in range(len(graph_df.index))}
+        works_info = set_connections_info(graph_df, history_df, mapper=name_mapper,
+                                          all_connections=all_connections,
+                                          change_connections_info=change_connections_info, id2ind=id2ind)
 
         return break_loops_in_input_graph(works_info)
 
+    @staticmethod
+    def work_graph(works_info: pd.DataFrame,
+                   name_mapper: NameMapper | None = None,
+                   work_resource_estimator: WorkTimeEstimator = DefaultWorkEstimator()) -> WorkGraph:
+        """
+        Gets a info about WorkGraph and Contractors from file .csv.
+
+        Schema of Contractors .csv file (optional data):
+            mandatory fields:
+                contractor_id: str - Id of the current contractor,
+                name: str - Contractor name as in the document
+            optional fields:
+                {names of resources}: float - each resource is a separate column
+
+        ATTENTION!
+            1) If you do not provide work resource estimator, framework uses built-in estimator
+
+
+        :param works_info: dataFrame that contains preprocessed info about work graph structure
+        :param contractor_types:
+        :param contractors_number: if we do not receive contractors, we need to know how many contractors the user wants,
+        for a generation
+        :param contractor_info: path to contractor info .csv file or list of Contractors
+        :param work_resource_estimator: work estimator that finds necessary resources, based on the history data
+        :return: WorkGraph and list of Contractors
+        """
+
+        works_info['activity_name_original'] = works_info.activity_name
+        if name_mapper:
+            works_info.activity_name = works_info.activity_name.apply(lambda name: name_mapper[name])
+
+        resources = [dict((worker_req.kind, int(worker_req.volume))
+                          for worker_req in work_resource_estimator.find_work_resources(w[0], float(w[1])))
+                     for w in works_info.loc[:, ['granular_name', 'volume']].to_numpy()]
+
+        unique_res = list(set(chain(*[r.keys() for r in resources])))
+        # works_info.loc[:, unique_res] = DataFrame(resources).fillna(0)
+
+        works_resources = add_graph_info(works_info)
+        works_resources = topsort_graph_df(works_resources)
+        work_graph = build_work_graph(works_resources, unique_res, work_resource_estimator)
+
+        return work_graph
+
     @staticmethod
     def work_graph_and_contractors(works_info: pd.DataFrame,
                                    contractor_info: str | list[Contractor] | tuple[ContractorGenerationMethod, int]
@@ -119,7 +143,7 @@ class CSVParser:
                                    contractor_types: list[int] | None = None,
                                    name_mapper: NameMapper | None = None,
                                    work_resource_estimator: WorkTimeEstimator = DefaultWorkEstimator()) \
-            -> (WorkGraph, Contractor):
+            -> tuple[WorkGraph, list[Contractor]]:
         """
         Gets a info about WorkGraph and Contractors from file .csv.
 
@@ -148,9 +172,6 @@ class CSVParser:
 
         # gather resources and contractors based on work resource estimator or contractor info .csv file
         contractors = []
-        works_info['activity_name_original'] = works_info.activity_name
-        if name_mapper:
-            works_info.activity_name = works_info.activity_name.apply(lambda name: name_mapper[name])
 
         if isinstance(contractor_info, list):
             contractors = contractor_info
@@ -170,16 +191,7 @@ class CSVParser:
                                equipments=dict())
                 )
 
-        resources = [dict((worker_req.kind, int(worker_req.volume))
-                          for worker_req in work_resource_estimator.find_work_resources(w[0], float(w[1])))
-                     for w in works_info.loc[:, ['activity_name', 'volume']].to_numpy()]
-
-        unique_res = list(set(chain(*[r.keys() for r in resources])))
-        # works_info.loc[:, unique_res] = DataFrame(resources).fillna(0)
-
-        works_resources = add_graph_info(works_info)
-        works_resources = topsort_graph_df(works_resources)
-        work_graph = build_work_graph(works_resources, unique_res, work_resource_estimator)
+        work_graph = CSVParser.work_graph(works_info, name_mapper, work_resource_estimator)
 
         # if we have no info about contractors or the user send an empty .csv file
         if len(contractors) == 0:
