@@ -2,8 +2,10 @@ import math
 from collections import defaultdict
 from typing import Callable, Any
 from uuid import uuid4
+from ast import literal_eval
 
 import networkx as nx
+import numpy as np
 import pandas as pd
 
 from sampo.schemas import Time
@@ -159,11 +161,21 @@ def preprocess_graph_df(frame: pd.DataFrame,
             if s.replace('.', '', 1).isdigit() \
             else s
 
+    temp_lst = [math.nan] * frame.shape[0]
+
+    for col in ['predecessor_ids', 'connection_types', 'lags', 'counts']:
+        if col not in frame.columns:
+            frame[col] = temp_lst
+
     if 'granular_name' not in frame.columns:
         frame['granular_name'] = [name_mapper[activity_name] for activity_name in frame['activity_name']]
 
     frame['activity_id'] = frame['activity_id'].astype(str)
-    frame['volume'] = frame['volume'].astype(float)
+    frame['volume'] = [float(x.replace(',', '.')) if isinstance(x, str) else float(x) for x in frame['volume']]
+
+    if 'min_req' in frame.columns and 'max_req' in frame.columns:
+        frame['min_req'] = [literal_eval(x) if isinstance(x, str) else x for x in frame['min_req']]
+        frame['max_req'] = [literal_eval(x) if isinstance(x, str) else x for x in frame['max_req']]
 
     frame['predecessor_ids'] = fix_df_column_with_arrays(frame['predecessor_ids'], cast=normalize_if_number)
     frame['connection_types'] = fix_df_column_with_arrays(frame['connection_types'],
@@ -172,6 +184,11 @@ def preprocess_graph_df(frame: pd.DataFrame,
     if 'lags' not in frame.columns:
         frame['lags'] = [NONE_ELEM] * len(frame)
     frame['lags'] = fix_df_column_with_arrays(frame['lags'], float)
+    for col in ['predecessor_ids', 'connection_types', 'lags', 'counts']:
+        frame[col] = frame[col].astype(object)
+
+    for _, row in frame.iterrows():
+        frame.at[_, 'counts'] = [np.iinfo(np.int64).max] * len(frame.at[_, 'lags'])
 
     return frame
 
@@ -219,22 +236,25 @@ def build_work_graph(frame: pd.DataFrame, resource_names: list[str], work_estima
     id_to_node = {}
 
     for _, row in frame.iterrows():
-        if 'min_req' in frame.columns and 'max_req' in frame.columns:
-            reqs = [WorkerReq(res_name, Time(int(row[res_name])),
-                              row['min_req'][res_name],
-                              row['max_req'][res_name]
-                              ) for res_name in resource_names
-                    if 0 < row['min_req'][res_name] <= row['max_req'][res_name]]
+        if 'min_req' in frame.columns and 'max_req' in frame.columns and 'req_volume' in frame.columns:
+            reqs = []
+            for res_name in resource_names:
+                if res_name in row['min_req'] and res_name in row['max_req']:
+                    if 0 < row['min_req'][res_name] <= row['max_req'][res_name]:
+                        reqs.append(WorkerReq(res_name, Time(int(row['req_volume'][res_name])),
+                                              row['min_req'][res_name],
+                                              row['max_req'][res_name]))
         else:
-            reqs = work_estimator.find_work_resources(row['activity_name'], float(row['volume']))
+            reqs = work_estimator.find_work_resources(row['granular_name'], float(row['volume']))
         is_service_unit = len(reqs) == 0
 
         zone_reqs = [ZoneReq(*v) for v in eval(row['required_statuses']).items()] \
             if 'required_statuses' in frame.columns else []
 
         description = row['description'] if 'description' in frame.columns else ''
+        group = row['group'] if 'group' in frame.columns else 'main project'
 
-        work_unit = WorkUnit(row['activity_id'], row['granular_name'], reqs, group=row['activity_name'],
+        work_unit = WorkUnit(row['activity_id'], row['granular_name'], reqs, group=group,
                              description=description, volume=row['volume'], volume_type=row['measurement'],
                              is_service_unit=is_service_unit, display_name=row['activity_name_original'],
                              zone_reqs=zone_reqs)
