@@ -5,6 +5,7 @@ import pandas as pd
 import plotly.express as px
 from matplotlib.figure import Figure
 
+from sampo.schemas import Time
 from sampo.utilities.visualization.base import VisualizationMode, visualize
 
 
@@ -33,6 +34,40 @@ def schedule_gant_chart_fig(schedule_dataframe: pd.DataFrame,
     visualization_start_delta = timedelta(days=2)
     visualization_finish_delta = timedelta(days=(schedule_finish - schedule_start).days // 3)
 
+    def create_delivery_row(i, mat_name, material) -> dict:
+        return {'idx': i,
+                'contractor': material[-1],
+                'cost': 0,
+                'volume': material[0],
+                'duration': 0,
+                'measurement': 'unit',
+                'workers_dict': '',
+                'workers': '',
+                'task_name_mapped': mat_name,
+                'task_name': '',
+                'zone_information': '',
+                'start': timedelta(material[1].value) + schedule_start,
+                'finish': timedelta(material[2].value) + schedule_start}
+
+    sworks = schedule_dataframe['scheduled_work_object'].copy()
+    idx = schedule_dataframe['idx'].copy()
+
+    def get_delivery_info(swork) -> str:
+        return '<br>' + '<br>'.join([f'{mat[0][0]}: {mat[0][1]}' for mat in swork.materials.delivery.values()])
+
+    schedule_dataframe['material_information'] = sworks.apply(get_delivery_info)
+
+    mat_delivery_row = []
+
+    # create material delivery information
+    for i, swork in zip(idx, sworks):
+        delivery = swork.materials.delivery
+        if not delivery:
+            mat_delivery_row.append(create_delivery_row(0, '', (0, Time(0), Time(0), '')))
+        for name, mat_info in delivery.items():
+            if delivery:
+                mat_delivery_row.append(create_delivery_row(i, name, mat_info[0]))
+
     def create_zone_row(i, zone_names, zone) -> dict:
         return {'idx': i,
                 'contractor': 'Access cards',
@@ -67,7 +102,7 @@ def schedule_gant_chart_fig(schedule_dataframe: pd.DataFrame,
         for zone in swork.zones_post:
             access_cards.append(create_zone_row(i, zone_names, zone))
 
-    schedule_dataframe = pd.concat([schedule_dataframe, pd.DataFrame.from_records(access_cards)])
+    schedule_dataframe = pd.concat([schedule_dataframe, pd.DataFrame.from_records(access_cards), pd.DataFrame.from_records(mat_delivery_row)])
 
     schedule_dataframe['color'] = schedule_dataframe[['task_name', 'contractor']] \
         .apply(lambda r: 'Defect' if ':' in r['task_name'] else r['contractor'], axis=1)
@@ -76,25 +111,17 @@ def schedule_gant_chart_fig(schedule_dataframe: pd.DataFrame,
                                                                      r['task_name'].split(':')[0]]['idx'].iloc[0]
                                  if ':' in r['task_name'] else r['idx'], axis=1))
 
-    # add one time unit to the end should remove hole within the immediately close tasks
-    schedule_dataframe['vis_finish'] = schedule_dataframe[['start', 'finish', 'duration']] \
-        .apply(lambda r: r['finish'] + timedelta(1) if r['duration'] > 0 else r['finish'], axis=1)
-    schedule_dataframe['vis_start'] = schedule_dataframe['start']
-    schedule_dataframe['finish'] = schedule_dataframe['finish'].apply(lambda x: x.strftime('%e %b %Y'))
-    schedule_dataframe['start'] = schedule_dataframe['start'].apply(lambda x: x.strftime('%e %b %Y'))
-
-    fig = px.timeline(schedule_dataframe, x_start='vis_start', x_end='vis_finish', y='idx', hover_name='task_name',
+    fig = px.timeline(schedule_dataframe, x_start='start', x_end='finish', y='idx', hover_name='task_name',
                       color=schedule_dataframe.loc[:, 'color'],
-                      hover_data={'vis_start': False,
-                                  'vis_finish': False,
-                                  'start': True,
+                      hover_data={'start': True,
                                   'finish': True,
                                   'task_name_mapped': True,
                                   'cost': True,
                                   'volume': True,
                                   'measurement': True,
                                   'workers': True,
-                                  'zone_information': True},
+                                  'zone_information': True,
+                                  'material_information': True},
                       title=f"{'Project tasks - Gant chart'}",
                       category_orders={'idx': list(schedule_dataframe.idx)},
                       text='task_name')
